/* OGMRip - A library for DVD ripping and encoding
 * Copyright (C) 2004-2007 Olivier Rolland <billl@users.sf.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "ogmrip-subp.h"

#define OGMRIP_SUBP_GET_PRIVATE(o) \
  (G_TYPE_INSTANCE_GET_PRIVATE ((o), OGMRIP_TYPE_SUBP, OGMRipSubpPriv))

struct _OGMRipSubpPriv
{
  OGMDvdSubpStream *stream;
  gboolean forced_only;
};

enum 
{
  PROP_0,
  PROP_STREAM,
  PROP_FORCED_ONLY
};

static void ogmrip_subp_finalize     (GObject      *gobject);
static void ogmrip_subp_set_property (GObject      *gobject,
                                       guint        property_id,
                                       const GValue *value,
                                       GParamSpec   *pspec);
static void ogmrip_subp_get_property (GObject      *gobject,
                                       guint        property_id,
                                       GValue       *value,
                                       GParamSpec   *pspec);

G_DEFINE_ABSTRACT_TYPE (OGMRipSubp, ogmrip_subp, OGMRIP_TYPE_CODEC)

static void
ogmrip_subp_class_init (OGMRipSubpClass *klass)
{
  GObjectClass *gobject_class;

  gobject_class = G_OBJECT_CLASS (klass);

  gobject_class->finalize = ogmrip_subp_finalize;
  gobject_class->set_property = ogmrip_subp_set_property;
  gobject_class->get_property = ogmrip_subp_get_property;

  g_object_class_install_property (gobject_class, PROP_STREAM, 
        g_param_spec_pointer ("stream", "Sub stream property", "Set subp stream", 
           G_PARAM_READWRITE));

  g_object_class_install_property (gobject_class, PROP_FORCED_ONLY, 
        g_param_spec_boolean ("forced-only", "Forced only property", "Set forced only", 
           FALSE, G_PARAM_READWRITE));

  g_type_class_add_private (klass, sizeof (OGMRipSubpPriv));
}

static void
ogmrip_subp_init (OGMRipSubp *subp)
{
  subp->priv = OGMRIP_SUBP_GET_PRIVATE (subp);
}

static void
ogmrip_subp_finalize (GObject *gobject)
{
  OGMRipSubp *subp;

  subp = OGMRIP_SUBP (gobject);
  if (subp->priv->stream)
    ogmdvd_subp_stream_unref (subp->priv->stream);
  subp->priv->stream = NULL;

  G_OBJECT_CLASS (ogmrip_subp_parent_class)->finalize (gobject);
}

static void
ogmrip_subp_set_property (GObject *gobject, guint property_id, const GValue *value, GParamSpec *pspec)
{
  OGMRipSubp *subp;

  subp = OGMRIP_SUBP (gobject);

  switch (property_id) 
  {
    case PROP_STREAM:
      ogmrip_subp_set_dvd_subp_stream (subp, g_value_get_pointer (value));
      break;
    case PROP_FORCED_ONLY: 
      subp->priv->forced_only = g_value_get_boolean (value);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
      break;
  }
}

static void
ogmrip_subp_get_property (GObject *gobject, guint property_id, GValue *value, GParamSpec *pspec)
{
  OGMRipSubp *subp;

  subp = OGMRIP_SUBP (gobject);

  switch (property_id) 
  {
    case PROP_STREAM:
      g_value_set_pointer (value, subp->priv->stream);
      break;
    case PROP_FORCED_ONLY: 
      g_value_set_boolean (value, subp->priv->forced_only);
      break;
    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
      break;
  }
}

void
ogmrip_subp_set_dvd_subp_stream (OGMRipSubp *subp, OGMDvdSubpStream *stream)
{
  g_return_if_fail (OGMRIP_IS_SUBP (subp));
  g_return_if_fail (stream != NULL);

  ogmdvd_subp_stream_ref (stream);

  if (subp->priv->stream)
    ogmdvd_subp_stream_unref (subp->priv->stream);
  subp->priv->stream = stream;

  ogmrip_codec_set_input (OGMRIP_CODEC (subp), 
      ogmdvd_subp_stream_get_title (stream));
}

OGMDvdSubpStream *
ogmrip_subp_get_dvd_subp_stream (OGMRipSubp *subp)
{
  g_return_val_if_fail (OGMRIP_IS_SUBP (subp), NULL);

  return subp->priv->stream;
}

void
ogmrip_subp_set_forced_only (OGMRipSubp *subp, gboolean forced_only)
{
  g_return_if_fail (OGMRIP_IS_SUBP (subp));

  subp->priv->forced_only = forced_only;
}

gboolean
ogmrip_subp_get_forced_only (OGMRipSubp *subp)
{
  g_return_val_if_fail (OGMRIP_IS_SUBP (subp), FALSE);

  return subp->priv->forced_only;
}

